from cvtech.HandTrackingModule import HandDetector
from cvtech.FaceTrackingModule import FaceDetector
from cvtech.FaceMeshModule import FaceMeshDetector
from cvtech.HumanTrackingModule import HumanDetector
from cvtech.HolisticModule import HolisticDetector
from cvtech.selfieSegmentation import SelfieSegmentation
